from kondo_ml.kondo_ml import KondoMLPackage

__author__ = "Lukas Rücker"
__email__ = "ruecker.lukas@gmail.com"
__version__ = "0.0.9"

# module level doc-string
__doc__ = """
Kondo-ML
================

Description
-----------
Kondo-ML is a Python package to gather instance selection algorithms for regression problems.
The package was created as a byproduct to my master's thesis. The name Kondo is referring to 
Marie Kondo. The idea is that with the instance selection only those instances are kept
that spark joy in the model (the user).


"""
